//
// Created by jie on 2023/10/11.
//

#include "texture.h"
#include "iostream"
#include "opencv2/opencv.hpp"
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp"
#include "glm/gtc/type_ptr.hpp"

constexpr int width = 800;
constexpr int height = 600;

#define STB_IMAGE_IMPLEMENTATION
using namespace std::literals::chrono_literals;
namespace TEXTURE {
    static float mixValue = 0.5f;

    void frameBufferSizeChange(GLFWwindow *window, int width, int height) {
        glViewport(0, 0, width, height);
    }

    void processInput(GLFWwindow *window) {
        if (glfwGetKey(window, GLFW_KEY_ESCAPE) == GLFW_PRESS) {
            glfwSetWindowShouldClose(window, true);
        }
        if (glfwGetKey(window, GLFW_KEY_UP) == GLFW_PRESS) {
            if (mixValue < 1.0) mixValue += 0.01f;
        }
        if (glfwGetKey(window, GLFW_KEY_DOWN) == GLFW_PRESS) {
            if (mixValue > 0) mixValue -= 0.01f;
        }
    }

    int Texture() {
        glfwInit();
        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
        glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

        GLFWwindow *window = glfwCreateWindow(width, height, "Windows", nullptr, nullptr);
        if (window == nullptr) {
            return -1;
        }
        glfwMakeContextCurrent(window);
        glfwSetWindowSizeCallback(window, frameBufferSizeChange);

        if (!gladLoadGLLoader((GLADloadproc) glfwGetProcAddress)) {
            auto error = glad_glGetError();
            return static_cast<int>(error);
        }

        float vertices[] = {
                0.5f, 0.5f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 1.0f,
                0.5f, -0.5f, 0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 0.0f,
                -0.5f, -0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
                -0.5f, 0.5f, 0.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f,
        };

        unsigned int indices[] = {
                0, 1, 3, // first triangle
                1, 2, 3  // second triangle
        };

        unsigned int VAO, VBO, EBO;
        glGenVertexArrays(1, &VAO);
        glGenBuffers(1, &VBO);
        glGenBuffers(1, &EBO);

        glBindVertexArray(VAO);
        glBindBuffer(GL_ARRAY_BUFFER, VBO);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, EBO);

        glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
        glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(indices), indices, GL_STATIC_DRAW);
        glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 8 * sizeof(float), (void *) 0);
        glEnableVertexAttribArray(0);
        glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 8 * sizeof(float), (void *) (3 * sizeof(float)));
        glEnableVertexAttribArray(1);
        glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, 8 * sizeof(float), (void *) (6 * sizeof(float)));
        glEnableVertexAttribArray(2);
        glBindVertexArray(0);
        auto wall = cv::imread("../../resources/container.jpg");
        auto awesomeFace = cv::imread("../../resources/awesomeface.png");
        cv::flip(awesomeFace, awesomeFace, 0);
        unsigned int texture[2];
        glGenTextures(2, texture);
        glBindTexture(GL_TEXTURE_2D, texture[0]);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, wall.cols, wall.rows, 0, GL_BGR, GL_UNSIGNED_BYTE, wall.data);
        glGenerateMipmap(GL_TEXTURE_2D);

        glBindTexture(GL_TEXTURE_2D, texture[1]);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, awesomeFace.cols, awesomeFace.rows, 0, GL_BGR, GL_UNSIGNED_BYTE,
                     awesomeFace.data);
        glGenerateMipmap(GL_TEXTURE_2D);

        auto shader = ShaderService("../../shaders/texture.vert",
                                    "../../shaders/texture.frag");

        shader.Use();
        shader.SetUniform<int>("ourTexture1", 0);
        shader.SetUniform<int>("ourTexture2", 1);

        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
        while (!glfwWindowShouldClose(window)) {
            processInput(window);

            glClearColor(.2f, .3f, .3f, 1.0f);
            glClear(GL_COLOR_BUFFER_BIT);
            glm::mat4 view = glm::mat4{1.f};
            glm::mat4 model = glm::mat4{1.f};
            glm::mat4 projection = glm::mat4{1.f};
            model = glm::rotate(model, glm::radians(-55.0f), {1.f, 0.f, 0.f});
            view = glm::translate(view, {0.f, 0.f, -3.f});
            projection = glm::perspective(glm::radians(45.f), static_cast<float>(width) / height, .1f, 100.f);

            shader.Use();
//            std::this_thread::sleep_for(5ms);
            shader.SetUniform("mixCount", mixValue);
            shader.SetUniform("model", model);
            shader.SetUniform("view", view);
            shader.SetUniform("projection", projection);

            glActiveTexture(GL_TEXTURE0);
            glBindTexture(GL_TEXTURE_2D, texture[0]);
            glActiveTexture(GL_TEXTURE1);
            glBindTexture(GL_TEXTURE_2D, texture[1]);

            glBindVertexArray(VAO);
            glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);
//            glUseProgram(shaderProgram[1]);
//            glDrawArrays(GL_TRIANGLES, 0, 3);
            glfwSwapBuffers(window);
            glfwPollEvents();
        }

        glDeleteBuffers(1, &VBO);
        glDeleteVertexArrays(1, &VAO);
        glDeleteBuffers(1, &EBO);

        glfwTerminate();
        return 0;
    }


    void MatrixTest() {
        glm::vec4 vec(1.f, 0.f, 0.f, 1.f);
        glm::mat4 trans = glm::mat4(1.0f);
        trans = glm::translate(trans, glm::vec3{1.f, 1.f, 0.f});
        vec = trans * vec;
        std::cout << vec.x << vec.y << vec.z << std::endl;
    }
}